{#
Renders Bokeh models into a basic .html file.

:param title: value for ``<title>`` tags
:type title: str

:param plot_resources: typically the output of RESOURCES
:type plot_resources: str

:param plot_script: typically the output of PLOT_SCRIPT
:type plot_script: str

Users can customize the file output by providing their own Jinja2 template
that accepts these same parameters.

#}

{% from macros import embed %}

{% block inner_head %}
  {% block preamble %}{% endblock %}
  {% block resources %}
    {% block js_resources %}
    {% endblock %}
  {% endblock %}
{% endblock %}
{% block postamble %}{% endblock %}
{% block body %}
  {% block inner_body %}
    {% block contents %}
      {% for doc in docs %}
        {{ embed(doc) if doc.elementid }}
        {% for root in doc.roots %}
          {% block root scoped %}
            {{ embed(root) | indent(10) }}
          {% endblock %}
        {% endfor %}
      {% endfor %}
    {% endblock %}
    {{ plot_script | indent(8) }}
  {% endblock %}
{% endblock %}
