body {
  color: white;
  background-color: #121212;
}

#header {
  background-color: #2f2f2f;
}

#main {
  color: white;
  background-color: #121212;
}

#sidebar {
  color: white;
  border-color: #2f2f2f !important;
  background-color: #121212;
}

.bk.card, .bk.accordion {
  color: white;
  background-color: #2f2f2f;
}

.bk.card-header, .bk.accordion-header {
  color: white;
  background-color: #292929 !important;
}

/* DataFrame Widget */
.bk-root .slick-header-columns {
  color: white;
  font-weight: bold;
  background: none;
}

.bk-root .slick-header-column:hover {
  background: #292929;
}

.bk-root .slick-row.even {
  background: none;
}

.bk-root .slick-row.odd {
  background: #3f3f3f;
}

.bk-root .slick-cell.selected {
  background: #3f3f80;
}

/* DataFrame Pane */
table.panel-df {
  color: white;
}

.panel-df tbody tr:nth-child(odd) {
  background: #3f3f3f;
}

.panel-df tr:hover {
    background: #3f3f80 !important;
    cursor: pointer;
}

/* Tabs layout */
.bk-root .bk-tabs-header .bk-tab {
  color: white;
  background: #2f2f2f;
  border: none;
}

.bk-root .bk-tabs-header .bk-tab:hover {
  color: white;
  background: #4f4f4f;
}

.bk-root .bk-tabs-header .bk-tab.bk-active {
  color: white;
  background: #3f3f3f;
  border: none;
}

html {
  scrollbar-face-color: #646464;
  scrollbar-base-color: #646464;
  scrollbar-3dlight-color: #646464;
  scrollbar-highlight-color: #646464;
  scrollbar-track-color: #000;
  scrollbar-arrow-color: #000;
  scrollbar-shadow-color: #646464;
  scrollbar-dark-shadow-color: #646464;
}

::-webkit-scrollbar-button {  background-color: #666; }
::-webkit-scrollbar-track {  background-color: #646464;}
::-webkit-scrollbar-track-piece { background-color: #000;}
::-webkit-scrollbar-thumb { height: 50px; background-color: #666; border-radius: 3px;}
::-webkit-scrollbar-corner { background-color: #646464;}}
::-webkit-resizer { background-color: #666;}

.bk.pn-loading:before {
  background-color: inherit;
  border-color: inherit;
}
