/* fast_root.css */
:root {
  --track-width: 2;
}
#menu * {
  width: 100%;
}
#menu ul {
  padding-left: 0px;
  padding-right: 35px;
}
#menu li {
  list-style-type: none;
  margin: 0;
  padding: 0;
}
#menu a {
  width: 100%;
  border-radius: calc(var(--corner-radius) * 1px);
  padding-left: 16px;
}
#menu a:hover {
  background: var(--accent-fill-hover);
  border-radius: calc(var(--corner-radius) * 1px);
}
#menu a:active {
  background: var(--accent-fill-checked);
  border-radius: calc(var(--corner-radius) * 1px);
}
#menu a[disabled=""] {
  pointer-events: none;
  color: var(--neutral-outline-rest)
}
h1, h2, h3, h4, h5, h6 {
  line-height: 1.5em;
}
::-webkit-scrollbar { width: 12px; height: 12px;}
#menu .menu-item-active a {
  background: var(--accent-fill-active);
  color: var(--accent-foreground-cut-rest);
}
