/* fast_template/dark.css */
#sidebar {
  border-right: 2px solid var(--neutral-fill-rest);
}

.bk.card-header {
  color: var(--neutral-foreground-rest);
  background-color: var(--neutral-fill-active) !important;
}

.bk.pn-loading:before {
  background-color: rgb(0,0,0,0.1);
}
